
#   this list is used by yalafi/tex2txt.py
#
load_table = {
    '*': [
        'amsmath',
        'amsthm',
        'babel',
        'biblatex',
        'circuitikz',
        # 'cleveref',
        # cleveref is not loaded by default because it throws
        # a warning if loaded without the poorman option.
        'geometry',
        'glossaries',
        'glossaries-extra',
        'graphicx',
        'hyperref',
        'inputenc',
        'listings',
        'mathtools',
        'pgfplots',
        'tikz',
        'unicode-math',
        'xcolor',
        'xspace',
    ],
    '': [
    ],
}
